//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

package ORG.opengroup.beans;

import java.io.*;
import java.util.*;
import java.beans.*;

/**
 * This bean supports debugging by class.
 */
final public class Debug implements Serializable
{
	private transient Class[] ca=null;
	private Vector v=new Vector();
	private static Debug debug=null;

	/**
	 * Checks to see if Class A implements or extends Class B
	 * @param a Class A
	 * @param b Class B
	 */
	 static boolean isSubclass(Class a, Class b)
	 {
	 	if (a==b)
	 		return true;

		Class x;
		for (x=a; x!=null; x=x.getSuperclass())
		{
			if (x==b)
				return true;
			if (a==null || b==null)
				return false;
			if (b.isInterface())
			{
				Class interfaces[]=x.getInterfaces();
				for (int i=0; i<interfaces.length;++i)
					if (isSubclass(interfaces[i],b))
						return true;
			}
		}

		return false;
	}

	/**
	 * Check if a class wants to be debugged
	 * @param c the class type
	 * @return true is the class is registered for debugging
	 *         false otherwise
	 */
	public static boolean isDebug(Class c)
	{
		if (debug==null)
			return false;
		int i;
		Class[] a=debug.ca;
		for (i=0;i<a.length;++i)
			if (isSubclass(c,a[i]))
				return true;
		return false;
	}

	/**
	 * Check if a class wants to be debugged
	 * @param s the component's name t
	 * @return true is the class is registered for debugging
	 *         false otherwise
	 */
	public static boolean isDebug(String name)
	{
		if (debug==null)
			return false;
		Class c=null;
		try
		{
			c=Class.forName(name);
		} catch (ClassNotFoundException e)
		{
			e.printStackTrace();
			return true;
		}
		return isDebug(c);
	}

	/**
	 * Check if a class wants to be debugged
	 * @param self the object to be debugged
	 * @return true is the class is registered for debugging
	 *         false otherwise
	 */
	public static boolean isDebug(Object self)
	{
		if (debug==null)
			return false;
		return isDebug(self.getClass());
	}

	/**
	 * Check if a class wants to be debugged
	 * @param self the object to be debugged
	 * @param line the statement to be printed if the self object is being debugged
	 * @return true is the class is registered for debugging
	 *         false otherwise
	 */
	public static void println(Object self, String line)
	{
		if (debug==null)
			return;
		if (isDebug(self.getClass()))
		    System.out.println(line);
	}

	/**
	 * The number of objects being debugged
	 */
	public int getSize()
	{
		return v.size();
	}

	/**
	 * The total number of objects that can be debugged
	 */
	public void setSize(int s)
        {
		if (s<v.size())
			v.setSize(s);
		while (s>v.size())
			v.addElement("");
        }

	/**
	 * The name of the class type of the object at index i
	 */
	public String getClass(int i)
		throws ArrayIndexOutOfBoundsException
	{
		return (String)v.elementAt(i);
	}

	/**
	 * The class type at index i in the vector containing the
	 * Debug classes
	 */
	public void setClass(int i, String s)
		throws ArrayIndexOutOfBoundsException
	{
		if (!Beans.isDesignTime())
			throw new Error("not design time");
		try
		{
			Class.forName(s);
		} catch (ClassNotFoundException e)
		{
			System.out.println("Debug: class["+i+"]="+s+"\n"+e.toString());
			System.exit(0);
		}
		v.setElementAt(s,i);
	}

	/**
	 * Default initialization
	 */
	public Debug()
	{
		init();
	}

	private void writeObject(ObjectOutputStream s)
		throws IOException
	{
		s.defaultWriteObject();
	}

	private void readObject(ObjectInputStream s)
		throws ClassNotFoundException, IOException
	{
		s.defaultReadObject();
		init();
	}

	/*
	 * This method gains control when
	 * the object is constructed or deserialized.
	 */
	void init()
	{
		if (!Beans.isDesignTime())
		{
			if (debug!=null)
			{
				System.out.println("Debug: multiple instances");
				System.exit(0);
			}
			debug=this;
			ca=new Class[v.size()];
			int i;
			for (i=0;i<ca.length;++i)
			{
				String n=(String)v.elementAt(i);
				if (n.length()==0)
					ca[i]=null;
				else
				{
					try
					{
						ca[i]=Class.forName(n);
					} catch (ClassNotFoundException e)
					{
						System.out.println("Debug: class["+i+"]="+n+"\n"+e.toString());
						System.exit(0);
					}
				}
			}
		}
	}
}
